@@ -6,6 +6,22 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
+// Copyright (C) 2007 Koji Otani <sho@bbr.jp>
+// Copyright (C) 2011 Albert Astals Cid <aacid@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef FOFITRUETYPE_H
 #define FOFITRUETYPE_H
 
@@ -13,6 +29,7 @@
 #pragma interface
 #endif
 
+#include "stddef.h"
 #include "goo/gtypes.h"
 #include "FoFiBase.h"
 
@@ -54,7 +71,7 @@ public:
   int findCmap(int platform, int encoding);
 
   // Return the GID corresponding to <c> according to the <i>th cmap.
-  Gushort mapCodeToGID(int i, int c);
+  Gushort mapCodeToGID(int i, Guint c);
 
   // map gid to vertical glyph gid if exist.
   //   if not exist return original gid
