@@ -27,7 +27,7 @@ pgd_info_add_permissions (GtkTable           *table,
 			  PopplerPermissions  permissions,
 			  gint               *row)
 {
-	GtkWidget *label, *hbox;
+	GtkWidget *label, *hbox, *vbox;
 	GtkWidget *checkbox;
 
 	label = gtk_label_new (NULL);
@@ -37,6 +37,7 @@ pgd_info_add_permissions (GtkTable           *table,
 			  GTK_FILL, GTK_FILL, 0, 0);
 	gtk_widget_show (label);
 
+        vbox = gtk_vbox_new (FALSE, 0);
 	hbox = gtk_hbox_new (FALSE, 6);
 
 	checkbox = gtk_check_button_new_with_label ("Print");
@@ -63,9 +64,41 @@ pgd_info_add_permissions (GtkTable           *table,
 	gtk_box_pack_start (GTK_BOX (hbox), checkbox, FALSE, TRUE, 0);
 	gtk_widget_show (checkbox);
 
-	gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, *row, *row + 1,
+        checkbox = gtk_check_button_new_with_label ("Fill forms");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox),
+                                      (permissions & POPPLER_PERMISSIONS_OK_TO_FILL_FORM));
+        gtk_box_pack_start (GTK_BOX (hbox), checkbox, FALSE, TRUE, 0);
+        gtk_widget_show (checkbox);
+
+        gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
+        gtk_widget_show (hbox);
+
+        hbox = gtk_hbox_new (FALSE, 6);
+
+        checkbox = gtk_check_button_new_with_label ("Extract contents");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox),
+                                      (permissions & POPPLER_PERMISSIONS_OK_TO_EXTRACT_CONTENTS));
+        gtk_box_pack_start (GTK_BOX (hbox), checkbox, FALSE, TRUE, 0);
+        gtk_widget_show (checkbox);
+
+        checkbox = gtk_check_button_new_with_label ("Assemble");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox),
+                                      (permissions & POPPLER_PERMISSIONS_OK_TO_ASSEMBLE));
+        gtk_box_pack_start (GTK_BOX (hbox), checkbox, FALSE, TRUE, 0);
+        gtk_widget_show (checkbox);
+
+        checkbox = gtk_check_button_new_with_label ("Print at high resolution");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox),
+                                      (permissions & POPPLER_PERMISSIONS_OK_TO_PRINT_HIGH_RESOLUTION));
+        gtk_box_pack_start (GTK_BOX (hbox), checkbox, FALSE, TRUE, 0);
+        gtk_widget_show (checkbox);
+
+        gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
+        gtk_widget_show (hbox);
+
+	gtk_table_attach (GTK_TABLE (table), vbox, 1, 2, *row, *row + 1,
 			  GTK_FILL, GTK_FILL, 0, 0);
-	gtk_widget_show (hbox);
+	gtk_widget_show (vbox);
 	
 	*row += 1;
 }
@@ -116,8 +149,11 @@ pgd_info_create_widget (PopplerDocument *document)
 	GtkWidget *frame, *alignment, *table;
 	gchar     *str;
 	gchar     *title, *format, *author, *subject;
-	gchar     *keywords, *creator, *producer, *linearized;
+	gchar     *keywords, *creator, *producer;
 	gchar     *metadata;
+	gchar     *perm_id;
+	gchar     *up_id;
+	gboolean   linearized;
 	GTime      creation_date, mod_date;
 	GEnumValue *enum_value;
 	PopplerBackend backend;
@@ -194,9 +230,8 @@ pgd_info_create_widget (PopplerDocument *document)
 	pgd_table_add_property (GTK_TABLE (table), "<b>Producer:</b>", producer, &row);
 	g_free (producer);
 	
-	pgd_table_add_property (GTK_TABLE (table), "<b>Linearized:</b>", linearized, &row);
-	g_free (linearized);
-	
+	pgd_table_add_property (GTK_TABLE (table), "<b>Linearized:</b>", linearized ? "Yes" : "No", &row);
+
 	str = pgd_format_date (creation_date);
 	pgd_table_add_property (GTK_TABLE (table), "<b>Creation Date:</b>", str, &row);
 	g_free (str);
@@ -211,6 +246,17 @@ pgd_info_create_widget (PopplerDocument *document)
 	enum_value = g_enum_get_value ((GEnumClass *) g_type_class_peek (POPPLER_TYPE_PAGE_LAYOUT), layout);
 	pgd_table_add_property (GTK_TABLE (table), "<b>Page Layout:</b>", enum_value->value_name, &row);
 
+	if (poppler_document_get_id (document, &perm_id, &up_id)) {
+		str = g_strndup (perm_id, 32);
+		g_free (perm_id);
+		pgd_table_add_property (GTK_TABLE (table), "<b>Permanent ID:</b>", str, &row);
+		g_free (str);
+		str = g_strndup (up_id, 32);
+		g_free (up_id);
+		pgd_table_add_property (GTK_TABLE (table), "<b>Update ID:</b>", str, &row);
+		g_free (str);
+	}
+
 	pgd_info_add_permissions (GTK_TABLE (table), permissions, &row);
 
 	pgd_info_add_metadata (GTK_TABLE (table), metadata, &row);
