@@ -19,6 +19,12 @@
 #include "poppler.h"
 #include "poppler-private.h"
 
+/**
+ * SECTION:poppler-action
+ * @short_description: Action links
+ * @title: PopplerAction
+ */
+
 POPPLER_DEFINE_BOXED_TYPE (PopplerDest, poppler_dest, poppler_dest_copy, poppler_dest_free)
 
 /**
@@ -34,8 +40,7 @@ poppler_dest_copy (PopplerDest *dest)
 {
 	PopplerDest *new_dest;
 
-	new_dest = g_new0 (PopplerDest, 1);
-	memcpy (new_dest, dest, sizeof (PopplerDest));
+	new_dest = g_slice_dup (PopplerDest, dest);
 
 	if (dest->named_dest)
 		new_dest->named_dest = g_strdup (dest->named_dest);
@@ -59,7 +64,7 @@ poppler_dest_free (PopplerDest *dest)
 	if (dest->named_dest)
 		g_free (dest->named_dest);
 	
-	g_free (dest);
+	g_slice_free (PopplerDest, dest);
 }
 
 static void
@@ -74,15 +79,14 @@ poppler_action_layer_free (PopplerActionLayer *action_layer)
 		action_layer->layers = NULL;
 	}
 
-	g_free (action_layer);
+	g_slice_free (PopplerActionLayer, action_layer);
 }
 
 static PopplerActionLayer *
 poppler_action_layer_copy (PopplerActionLayer *action_layer)
 {
-	PopplerActionLayer *retval = g_new (PopplerActionLayer, 1);
+	PopplerActionLayer *retval = g_slice_dup (PopplerActionLayer, action_layer);
 
-	retval->action = action_layer->action;
 	retval->layers = g_list_copy (action_layer->layers);
 	g_list_foreach (action_layer->layers, (GFunc)g_object_ref, NULL);
 
@@ -136,12 +140,16 @@ poppler_action_free (PopplerAction *action)
 			g_list_free (action->ocg_state.state_list);
 		}
 		break;
+	case POPPLER_ACTION_JAVASCRIPT:
+		if (action->javascript.script)
+			g_free (action->javascript.script);
+		break;
 	default:
 		break;
 	}
 	
 	g_free (action->any.title);
-	g_free (action);
+	g_slice_free (PopplerAction, action);
 }
 
 /**
@@ -160,8 +168,7 @@ poppler_action_copy (PopplerAction *action)
 	g_return_val_if_fail (action != NULL, NULL);
 
 	/* Do a straight copy of the memory */
-	new_action = g_new0 (PopplerAction, 1);
-	memcpy (new_action, action, sizeof (PopplerAction));
+	new_action = g_slice_dup (PopplerAction, action);
 
 	if (action->any.title != NULL)
 		new_action->any.title = g_strdup (action->any.title);
@@ -211,6 +218,10 @@ poppler_action_copy (PopplerAction *action)
 		}
 
 		break;
+	case POPPLER_ACTION_JAVASCRIPT:
+		if (action->javascript.script)
+			new_action->javascript.script = g_strdup (action->javascript.script);
+		break;
 	default:
 		break;
 	}
@@ -224,7 +235,7 @@ dest_new_goto (PopplerDocument *document,
 {
 	PopplerDest *dest;
 
-	dest = g_new0 (PopplerDest, 1);
+	dest = g_slice_new0 (PopplerDest);
 
 	if (link_dest == NULL) {
 		dest->type = POPPLER_DEST_UNKNOWN;
@@ -309,7 +320,7 @@ dest_new_named (GooString *named_dest)
 {
 	PopplerDest *dest;
 
-	dest = g_new0 (PopplerDest, 1);
+	dest = g_slice_new0 (PopplerDest);
 
 	if (named_dest == NULL) {
 		dest->type = POPPLER_DEST_UNKNOWN;
@@ -422,13 +433,13 @@ find_annot_movie_for_action (PopplerDocument *document,
 
     xref->fetch (ref->num, ref->gen, &annotObj);
   } else if (link->hasAnnotTitle ()) {
-    Catalog *catalog = document->doc->getCatalog ();
     Object annots;
     GooString *title = link->getAnnotTitle ();
     int i;
 
     for (i = 1; i <= document->doc->getNumPages (); ++i) {
-      Page *p = catalog->getPage (i);
+      Page *p = document->doc->getPage (i);
+      if (!p) continue;
 
       if (p->getAnnots (&annots)->isArray ()) {
         int j;
@@ -510,6 +521,18 @@ build_movie (PopplerDocument *document,
 	}
 }
 
+static void
+build_javascript (PopplerAction *action,
+		  LinkJavaScript *link)
+{
+	GooString *script;
+
+	script = link->getScript();
+	if (script)
+		action->javascript.script = _poppler_goo_string_to_utf8 (script);
+
+}
+
 static void
 build_rendition (PopplerAction *action,
 		 LinkRendition *link)
@@ -604,7 +627,7 @@ _poppler_action_new (PopplerDocument *document,
 {
 	PopplerAction *action;
 
-	action = g_new0 (PopplerAction, 1);
+	action = g_slice_new0 (PopplerAction);
 
 	if (title)
 		action->any.title = g_strdup (title);
@@ -647,6 +670,10 @@ _poppler_action_new (PopplerDocument *document,
 		action->type = POPPLER_ACTION_OCG_STATE;
 		build_ocg_state (document, action, dynamic_cast<LinkOCGState*> (link));
 		break;
+	case actionJavaScript:
+		action->type = POPPLER_ACTION_JAVASCRIPT;
+		build_javascript (action, dynamic_cast<LinkJavaScript*> (link));
+		break;
 	case actionUnknown:
 	default:
 		action->type = POPPLER_ACTION_UNKNOWN;
