@@ -16,6 +16,7 @@
  * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "config.h"
 #include <string.h>
 
 #ifndef __GI_SCANNER__
@@ -475,6 +476,29 @@ poppler_document_get_page_by_label (PopplerDocument  *document,
   return poppler_document_get_page (document, index);
 }
 
+/**
+ * poppler_document_get_n_attachments:
+ * @document: A #PopplerDocument
+ *
+ * Returns the number of attachments in a loaded document.
+ * See also poppler_document_get_attachments()
+ *
+ * Return value: Number of attachments
+ *
+ * Since: 0.18
+ */
+guint
+poppler_document_get_n_attachments (PopplerDocument *document)
+{
+  Catalog *catalog;
+
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), 0);
+
+  catalog = document->doc->getCatalog ();
+
+  return catalog && catalog->isOk () ? catalog->numEmbeddedFiles () : 0;
+}
+
 /**
  * poppler_document_has_attachments:
  * @document: A #PopplerDocument
@@ -486,18 +510,9 @@ poppler_document_get_page_by_label (PopplerDocument  *document,
 gboolean
 poppler_document_has_attachments (PopplerDocument *document)
 {
-  Catalog *catalog;
-  int n_files = 0;
-
   g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), FALSE);
 
-  catalog = document->doc->getCatalog ();
-  if (catalog && catalog->isOk ())
-    {
-      n_files = catalog->numEmbeddedFiles ();
-    }
-
-  return (n_files != 0);
+  return (poppler_document_get_n_attachments (document) != 0);
 }
 
 /**
@@ -527,13 +542,14 @@ poppler_document_get_attachments (PopplerDocument *document)
   for (i = 0; i < n_files; i++)
     {
       PopplerAttachment *attachment;
-      EmbFile *emb_file;
+      FileSpec *emb_file;
 
       emb_file = catalog->embeddedFile (i);
-      if (!emb_file->isOk ()) {
+      if (!emb_file->isOk () || !emb_file->getEmbeddedFile()->isOk()) {
         delete emb_file;
 	continue;
       }
+
       attachment = _poppler_attachment_new (emb_file);
       delete emb_file;
 
@@ -715,10 +731,10 @@ poppler_document_get_pdf_version_string (PopplerDocument *document)
 /**
  * poppler_document_get_pdf_version:
  * @document: A #PopplerDocument
- * @major_version: (allow-none): return location for the PDF major version number
- * @minor_version: (allow-none): return location for the PDF minor version number
+ * @major_version: (out) (allow-none): return location for the PDF major version number
+ * @minor_version: (out) (allow-none): return location for the PDF minor version number
  *
- * Returns the major and minor PDF version numbers.
+ * Returns: the major and minor PDF version numbers
  *
  * Since: 0.16
  **/
@@ -1048,6 +1064,12 @@ poppler_document_get_permissions (PopplerDocument *document)
     flag |= POPPLER_PERMISSIONS_OK_TO_ADD_NOTES;
   if (document->doc->okToFillForm ())
     flag |= POPPLER_PERMISSIONS_OK_TO_FILL_FORM;
+  if (document->doc->okToAccessibility())
+    flag |= POPPLER_PERMISSIONS_OK_TO_EXTRACT_CONTENTS;
+  if (document->doc->okToAssemble())
+    flag |= POPPLER_PERMISSIONS_OK_TO_ASSEMBLE;
+  if (document->doc->okToPrintHighRes())
+    flag |= POPPLER_PERMISSIONS_OK_TO_PRINT_HIGH_RESOLUTION;
 
   return (PopplerPermissions)flag;
 }
@@ -2298,11 +2320,11 @@ poppler_layers_iter_get_title (PopplerLayersIter *iter)
 /**
  * poppler_layers_iter_get_layer:
  * @iter: a #PopplerLayersIter
- * 
- * Returns the #PopplerLayer associated with @iter.  It must be freed with
- * poppler_layer_free().
- * 
- * Return value: a new #PopplerLayer, or %NULL if there isn't any layer associated with @iter
+ *
+ * Returns the #PopplerLayer associated with @iter.
+ *
+ * Return value: (transfer full): a new #PopplerLayer, or %NULL if
+ * there isn't any layer associated with @iter
  *
  * Since: 0.12
  **/
@@ -2475,7 +2497,8 @@ poppler_ps_file_free (PopplerPSFile *ps_file)
  * Returns the #PopplerFormField for the given @id. It must be freed with
  * g_object_unref()
  *
- * Return value: a new #PopplerFormField or NULL if not found
+ * Return value: (transfer full): a new #PopplerFormField or %NULL if
+ * not found
  **/
 PopplerFormField *
 poppler_document_get_form_field (PopplerDocument *document,
@@ -2493,7 +2516,7 @@ poppler_document_get_form_field (PopplerDocument *document,
   if (!page)
     return NULL;
 
-  widgets = page->getPageWidgets ();
+  widgets = page->getFormWidgets (document->doc->getCatalog ());
   if (!widgets)
     return NULL;
 
