@@ -18,7 +18,8 @@
 // Copyright (C) 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2006 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
 // Copyright (C) 2007 Jeff Muizelaar <jeff@infidigm.net>
-// Copyright (C) 2008-2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008-2011 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2011 Kenji Uno <ku@digitaldolphins.jp>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -129,6 +130,8 @@ void inline GooString::resize(int newLength) {
       } else {
 	memcpy(s1, s, length);
       }
+      if (s != sStatic)
+	gfree(s);
     }
 
   }
@@ -238,7 +241,7 @@ GooString *GooString::formatv(char *fmt, va_list argList) {
 
 GooString::~GooString() {
   if (s != sStatic)
-    free(s);
+    gfree(s);
 }
 
 GooString *GooString::clear() {
