@@ -6,6 +6,8 @@
 //
 // Copyright 2005 Jonathan Blandford <jrb@redhat.com>
 // Copyright 2007 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
+// Copyright 2010 Hib Eris <hib@hiberis.nl>
+// Copyright 2011 Albert Astals cid <aacid@kde.org>
 // Inspired by gtimer.c in glib, which is Copyright 2000 by the GLib Team
 //
 //========================================================================
@@ -17,12 +19,13 @@
 #pragma interface
 #endif
 
+#include "poppler/poppler-config.h"
 #include "gtypes.h"
 #ifdef HAVE_GETTIMEOFDAY
 #include <sys/time.h>
 #endif
 
-#ifdef _MSC_VER
+#ifdef _WIN32
 #include <windows.h>
 #endif
 
@@ -44,7 +47,7 @@ private:
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval start_time;
   struct timeval end_time;
-#elif defined(_MSC_VER)
+#elif defined(_WIN32)
   LARGE_INTEGER start_time;
   LARGE_INTEGER end_time;
 #endif
