@@ -18,13 +18,14 @@
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2005 Martin Kretzschmar <martink@gnome.org>
 // Copyright (C) 2005, 2009 Albert Astals Cid <aacid@kde.org>
-// Copyright (C) 2006, 2007, 2010 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2006, 2007, 2010, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2007 Koji Otani <sho@bbr.jp>
 // Copyright (C) 2008, 2009 Chris Wilson <chris@chris-wilson.co.uk>
 // Copyright (C) 2008 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2009 Darren Kenny <darren.kenny@sun.com>
 // Copyright (C) 2010 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
 // Copyright (C) 2010 Jan Kümmel <jan+freedesktop@snorc.org>
+// Copyright (C) 2012 Hib Eris <hib@hiberis.nl>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -106,7 +107,7 @@ CairoFont::getGlyph(CharCode code,
 double
 CairoFont::getSubstitutionCorrection(GfxFont *gfxFont)
 {
-  double w1, w2,w3;
+  double w1, w2;
   CharCode code;
   char *name;
 
@@ -134,7 +135,6 @@ CairoFont::getSubstitutionCorrection(GfxFont *gfxFont)
 
 	cairo_scaled_font_destroy(scaled_font);
 	cairo_font_options_destroy(options);
-	w3 = extents.width;
 	w2 = extents.x_advance;
       }
       if (!gfxFont->isSymbolic()) {
@@ -467,8 +467,8 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
       }
     }
     break;
-    
   case fontCIDType2:
+  case fontCIDType2OT:
     codeToGID = NULL;
     n = 0;
     if (((GfxCIDFont *)gfxFont)->getCIDToGID()) {
@@ -707,7 +707,7 @@ CairoType3Font *CairoType3Font::create(GfxFont *gfxFont, XRef *xref,
 
 CairoType3Font::CairoType3Font(Ref ref,
 			       XRef *xref,
-			       Catalog *cat,
+			       Catalog *catalog,
 			       cairo_font_face_t *cairo_font_face,
 			       Gushort *codeToGID,
 			       Guint codeToGIDLen,
