@@ -2,13 +2,15 @@
 //
 // FlateStream.h
 //
-// Copyright (C) 2005, Jeff Muizelaar
+// Copyright (C) 2005, Jeff Muizelaar <jeff@infidigm.net>
+// Copyright (C) 2010, 2011, Albert Astals Cid <aacid@kde.org>
+//
+// This file is under the GPLv2 or later license
 //
 //========================================================================
 
 #ifndef FLATESTREAM_H
 #define FLATESTREAM_H
-#include <config.h>
 
 #ifdef USE_GCC_PRAGMAS
 #pragma interface
@@ -19,17 +21,17 @@
 #pragma implementation
 #endif
 
+#include "poppler-config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
-#ifndef WIN32
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <string.h>
 #include <ctype.h>
 #include "goo/gmem.h"
 #include "goo/gfile.h"
-#include "poppler-config.h"
 #include "Error.h"
 #include "Object.h"
 #include "Decrypt.h"
@@ -49,15 +51,24 @@ public:
   virtual int getChar();
   virtual int lookChar();
   virtual int getRawChar();
+  virtual void getRawChars(int nChars, int *buffer);
   virtual GooString *getPSFilter(int psLevel, char *indent);
   virtual GBool isBinary(GBool last = gTrue);
 
 private:
+  inline int doGetRawChar() {
+    if (fill_buffer())
+      return EOF;
+
+    return out_buf[out_pos++];
+  }
+
   int fill_buffer(void);
   z_stream d_stream;
   StreamPredictor *pred;
   int status;
-  unsigned char in_buf[4096];
+  /* in_buf currently needs to be 1 or we over read from EmbedStreams */
+  unsigned char in_buf[1];
   unsigned char out_buf[4096];
   int out_pos;
   int out_buf_len;
