@@ -15,6 +15,8 @@
 //
 // Copyright (C) 2006, 2008-2010 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006 Jeff Muizelaar <jeff@infidigm.net>
+// Copyright (C) 2010 Christian Feuersnger <cfeuersaenger@googlemail.com>
+// Copyright (C) 2011 Andrea Canciani <ranma42@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -38,7 +40,6 @@
 #include "Stream.h"
 #include "Error.h"
 #include "Function.h"
-#include "PopplerCache.h"
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
@@ -528,6 +529,7 @@ ExponentialFunction::ExponentialFunction(Object *funcObj, Dict *dict) {
   e = obj1.getNum();
   obj1.free();
 
+  isLinear = fabs(e-1.) < 1e-10;
   ok = gTrue;
   return;
 
@@ -558,7 +560,7 @@ void ExponentialFunction::transform(double *in, double *out) {
     x = in[0];
   }
   for (i = 0; i < n; ++i) {
-    out[i] = c0[i] + pow(x, e) * (c1[i] - c0[i]);
+    out[i] = c0[i] + (isLinear ? x : pow(x, e)) * (c1[i] - c0[i]);
     if (hasRange) {
       if (out[i] < range[i][0]) {
 	out[i] = range[i][0];
@@ -676,6 +678,7 @@ StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, std::set<int>
     }
   }
 
+  n = funcs[0]->getOutputSize();
   ok = gTrue;
   return;
 
@@ -1046,84 +1049,6 @@ void PSStack::roll(int n, int j) {
   }
 }
 
-class PostScriptFunctionKey : public PopplerCacheKey
-{
-  public:
-    PostScriptFunctionKey(int sizeA, double *inA, bool copyA)
-    {
-      init(sizeA, inA, copyA);
-    }
-    
-    PostScriptFunctionKey(const PostScriptFunctionKey &key)
-    {
-      init(key.size, key.in, key.copied);
-    }
-    
-    void init(int sizeA, double *inA, bool copyA)
-    {
-      copied = copyA;
-      size = sizeA;
-      if (copied) {
-        in = new double[size];
-        for (int i = 0; i < size; ++i) in[i] = inA[i];
-      } else {
-        in = inA;
-      }
-    }
-    
-    ~PostScriptFunctionKey()
-    {
-      if (copied) delete[] in;
-    }
-       
-    bool operator==(const PopplerCacheKey &key) const
-    {
-      const PostScriptFunctionKey *k = static_cast<const PostScriptFunctionKey*>(&key);
-      if (size == k->size) {
-        bool equal = true;
-        for (int i = 0; equal && i < size; ++i) {
-          equal = in[i] == k->in[i];
-        }
-        return equal;
-      } else {
-        return false;
-      }
-    }
-  
-    bool copied;
-    int size;
-    double *in;
-};
-
-class PostScriptFunctionItem : public PopplerCacheItem
-{
-  public:
-    PostScriptFunctionItem(int sizeA, double *outA)
-    {
-      init(sizeA, outA);
-    }
-    
-    PostScriptFunctionItem(const PostScriptFunctionItem &item)
-    {
-      init(item.size, item.out);
-    }
-    
-    void init(int sizeA, double *outA)
-    {
-      size = sizeA;
-      out = new double[size];
-      for (int i = 0; i < size; ++i) out[i] = outA[i];
-    }
-    
-    ~PostScriptFunctionItem()
-    {
-      delete[] out;
-    }
-    
-    int size;
-    double *out;
-};
-
 PostScriptFunction::PostScriptFunction(Object *funcObj, Dict *dict) {
   Stream *str;
   int codePtr;
@@ -1132,9 +1057,7 @@ PostScriptFunction::PostScriptFunction(Object *funcObj, Dict *dict) {
   code = NULL;
   codeString = NULL;
   codeSize = 0;
-  stack = NULL;
   ok = gFalse;
-  cache = new PopplerCache(5);
 
   //----- initialize the generic stuff
   if (!init(dict)) {
@@ -1171,8 +1094,6 @@ PostScriptFunction::PostScriptFunction(Object *funcObj, Dict *dict) {
 
   ok = gTrue;
   
-  stack = new PSStack();
-
  err2:
   str->close();
  err1:
@@ -1184,58 +1105,33 @@ PostScriptFunction::PostScriptFunction(PostScriptFunction *func) {
   code = (PSObject *)gmallocn(codeSize, sizeof(PSObject));
   memcpy(code, func->code, codeSize * sizeof(PSObject));
   codeString = func->codeString->copy();
-  stack = new PSStack();
-  memcpy(stack, func->stack, sizeof(PSStack));
-  
-  cache = new PopplerCache(func->cache->size());
-  for (int i = 0; i < func->cache->numberOfItems(); ++i)
-  {
-    PostScriptFunctionKey *key = new PostScriptFunctionKey(*(PostScriptFunctionKey*)func->cache->key(i));
-    PostScriptFunctionItem *item = new PostScriptFunctionItem(*(PostScriptFunctionItem*)func->cache->item(i));
-    cache->put(key, item);
-  }
 }
 
 PostScriptFunction::~PostScriptFunction() {
   gfree(code);
   delete codeString;
-  delete stack;
-  delete cache;
 }
 
 void PostScriptFunction::transform(double *in, double *out) {
+  PSStack stack;
   int i;
   
-  PostScriptFunctionKey key(m, in, false);
-  PopplerCacheItem *item = cache->lookup(key);
-  if (item) {
-    PostScriptFunctionItem *it = static_cast<PostScriptFunctionItem *>(item);
-    for (int i = 0; i < n; ++i) {
-      out[i] = it->out[i];
-    }
-    return;
-  }
-
-  stack->clear();
   for (i = 0; i < m; ++i) {
     //~ may need to check for integers here
-    stack->pushReal(in[i]);
+    stack.pushReal(in[i]);
   }
-  exec(stack, 0);
+  exec(&stack, 0);
   for (i = n - 1; i >= 0; --i) {
-    out[i] = stack->popNum();
+    out[i] = stack.popNum();
     if (out[i] < range[i][0]) {
       out[i] = range[i][0];
     } else if (out[i] > range[i][1]) {
       out[i] = range[i][1];
     }
   }
+  stack.clear();
 
-  PostScriptFunctionKey *newKey = new PostScriptFunctionKey(m, in, true);
-  PostScriptFunctionItem *newItem = new PostScriptFunctionItem(n, out);
-  cache->put(newKey, newItem);
-  
-  // if (!stack->empty()) {
+  // if (!stack.empty()) {
   //   error(-1, "Extra values on stack at end of PostScript function");
   // }
 }
