@@ -6,6 +6,27 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2005, 2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
+// Copyright (C) 2006 Kristian Høgsberg <krh@redhat.com>
+// Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
+// Copyright (C) 2007 Jeff Muizelaar <jeff@infidigm.net>
+// Copyright (C) 2007 Koji Otani <sho@bbr.jp>
+// Copyright (C) 2011 Axel Strübing <axel.struebing@freenet.de>
+// Copyright (C) 2011 Adrian Johnson <ajohnson@redneon.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef GFXFONT_H
 #define GFXFONT_H
 
@@ -116,8 +137,6 @@ public:
 
   GfxFont(char *tagA, Ref idA, GooString *nameA);
 
-  virtual ~GfxFont();
-
   GBool isOk() { return ok; }
 
   void incRefCnt();
@@ -200,9 +219,12 @@ public:
   // the number actually used.  Returns the number of bytes used by
   // the char code.
   virtual int getNextChar(char *s, int len, CharCode *code,
-			  Unicode *u, int uSize, int *uLen,
+			  Unicode **u, int *uLen,
 			  double *dx, double *dy, double *ox, double *oy) = 0;
 
+  // Does this font have a toUnicode map?
+  GBool hasToUnicodeCMap() { return hasToUnicode; }
+
   /* XXX: dfp shouldn't be public, however the font finding code is currently in
    * GlobalParams. Instead it should be inside the GfxFont class. However,
    * getDisplayFont currently uses FCcfg so moving it is not as simple. */
@@ -210,6 +232,8 @@ public:
   DisplayFontParam *dfp;
 protected:
 
+  virtual ~GfxFont();
+
   void readFontDescriptor(XRef *xref, Dict *fontDict);
   CharCodeToUnicode *readToUnicodeCMap(Dict *fontDict, int nBits,
 				       CharCodeToUnicode *ctu);
@@ -234,6 +258,7 @@ protected:
   double descent;		// max depth below baseline
   int refCnt;
   GBool ok;
+  GBool hasToUnicode;
 };
 
 //------------------------------------------------------------------------
@@ -246,10 +271,8 @@ public:
   Gfx8BitFont(XRef *xref, char *tagA, Ref idA, GooString *nameA,
 	      GfxFontType typeA, Dict *fontDict);
 
-  virtual ~Gfx8BitFont();
-
   virtual int getNextChar(char *s, int len, CharCode *code,
-			  Unicode *u, int uSize, int *uLen,
+			  Unicode **u, int *uLen,
 			  double *dx, double *dy, double *ox, double *oy);
 
   // Return the encoding.
@@ -284,6 +307,7 @@ public:
   Dict *getResources();
 
 private:
+  virtual ~Gfx8BitFont();
 
   char *enc[256];		// char code --> char name
   char encFree[256];		// boolean for each char name: if set,
@@ -306,12 +330,10 @@ public:
   GfxCIDFont(XRef *xref, char *tagA, Ref idA, GooString *nameA,
 	     Dict *fontDict);
 
-  virtual ~GfxCIDFont();
-
   virtual GBool isCIDFont() { return gTrue; }
 
   virtual int getNextChar(char *s, int len, CharCode *code,
-			  Unicode *u, int uSize, int *uLen,
+			  Unicode **u, int *uLen,
 			  double *dx, double *dy, double *ox, double *oy);
 
   // Return the writing mode (0=horizontal, 1=vertical).
@@ -333,6 +355,8 @@ public:
   double getWidth(char* s, int len);
 
 private:
+  virtual ~GfxCIDFont();
+
   Gushort mapCodeToGID(FoFiTrueType *ff, int cmapi,
     Unicode unicode, GBool wmode);
 
