@@ -6,6 +6,23 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2006-2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
+// Copyright (C) 2010 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2012 Adrian Johnson <ajohnson@redneon.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include <config.h>
 
 #ifdef USE_GCC_PRAGMAS
@@ -15,6 +32,7 @@
 #include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
+#include <limits.h>
 #include <ctype.h>
 #include "Lexer.h"
 #include "Error.h"
@@ -24,7 +42,7 @@
 
 // A '1' in this array means the character is white space.  A '1' or
 // '2' means the character ends a name or command.
-static char specialChars[256] = {
+static const char specialChars[256] = {
   1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0,   // 0x
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   // 1x
   1, 0, 0, 0, 0, 2, 0, 0, 2, 2, 0, 0, 0, 0, 0, 2,   // 2x
@@ -43,6 +61,8 @@ static char specialChars[256] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0    // fx
 };
 
+static const int IntegerSafeLimit = (INT_MAX - 9) / 10;
+
 //------------------------------------------------------------------------
 // Lexer
 //------------------------------------------------------------------------
@@ -135,10 +155,11 @@ int Lexer::lookChar() {
 Object *Lexer::getObj(Object *obj, int objNum) {
   char *p;
   int c, c2;
-  GBool comment, neg, done;
+  GBool comment, neg, done, overflownInteger, overflownUnsignedInteger;
   int numParen;
   int xi;
-  double xf, scale;
+  unsigned int xui = 0;
+  double xf = 0, scale;
   GooString *s;
   int n, m;
 
@@ -164,21 +185,43 @@ Object *Lexer::getObj(Object *obj, int objNum) {
   // number
   case '0': case '1': case '2': case '3': case '4':
   case '5': case '6': case '7': case '8': case '9':
-  case '-': case '.':
+  case '+': case '-': case '.':
+    overflownInteger = gFalse;
+    overflownUnsignedInteger = gFalse;
     neg = gFalse;
     xi = 0;
     if (c == '-') {
       neg = gTrue;
     } else if (c == '.') {
       goto doReal;
-    } else {
+    } else if (c != '+') {
       xi = c - '0';
     }
     while (1) {
       c = lookChar();
       if (isdigit(c)) {
 	getChar();
-	xi = xi * 10 + (c - '0');
+	if (unlikely(overflownInteger)) {
+	  if (overflownUnsignedInteger) {
+	    xf = xf * 10.0 + (c - '0');
+	  } else {
+	    overflownUnsignedInteger = gTrue;
+	    xf = xui * 10.0 + (c - '0');
+	  }
+	} else {
+	  if (unlikely(xi > IntegerSafeLimit) &&
+	      (xi > (INT_MAX - (c - '0')) / 10.0)) {
+	    overflownInteger = gTrue;
+	    if (xi > (UINT_MAX - (c - '0')) / 10.0) {
+	      overflownUnsignedInteger = gTrue;
+	      xf = xi * 10.0 + (c - '0');
+	    } else {
+	      xui = xi * 10.0 + (c - '0');
+	    }
+	  } else {
+	    xi = xi * 10 + (c - '0');
+	  }
+	}
       } else if (c == '.') {
 	getChar();
 	goto doReal;
@@ -186,12 +229,26 @@ Object *Lexer::getObj(Object *obj, int objNum) {
 	break;
       }
     }
-    if (neg)
+    if (neg) {
       xi = -xi;
-    obj->initInt(xi);
+      xf = -xf;
+    }
+    if (unlikely(overflownInteger)) {
+      if (overflownUnsignedInteger) {
+        obj->initReal(xf);
+      } else {
+        obj->initUint(xui);
+      }
+    } else {
+      obj->initInt(xi);
+    }
     break;
   doReal:
-    xf = xi;
+    if (likely(!overflownInteger)) {
+      xf = xi;
+    } else if (!overflownUnsignedInteger) {
+      xf = xui;
+    }
     scale = 0.1;
     while (1) {
       c = lookChar();
