@@ -15,11 +15,11 @@
 //
 // Copyright (C) 2005 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2005 Jeff Muizelaar <jeff@infidigm.net>
-// Copyright (C) 2005-2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2011 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006-2008 Pino Toscano <pino@kde.org>
 // Copyright (C) 2006 Nickolay V. Shmyrev <nshmyrev@yandex.ru>
 // Copyright (C) 2006 Scott Turner <scotty1024@mac.com>
-// Copyright (C) 2006-2009 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2006-2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2008 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright (C) 2008 Brad Hards <bradh@kde.org>
@@ -45,13 +45,11 @@
 #include "XRef.h"
 #include "Link.h"
 #include "OutputDev.h"
-#ifndef PDF_PARSER_ONLY
 #include "Gfx.h"
 #include "GfxState.h"
 #include "Annot.h"
 #include "TextOutputDev.h"
 #include "Form.h"
-#endif
 #include "Error.h"
 #include "Page.h"
 #include "Catalog.h"
@@ -262,7 +260,7 @@ Page::Page(XRef *xrefA, int numA, Dict *pageDict, Ref pageRefA, PageAttrs *attrs
   xref = xrefA;
   num = numA;
   duration = -1;
-  pageWidgets = NULL;
+  annots = NULL;
 
   pageObj.initDict(pageDict);
   pageRef = pageRefA;
@@ -289,18 +287,14 @@ Page::Page(XRef *xrefA, int numA, Dict *pageDict, Ref pageRefA, PageAttrs *attrs
   tmp.free();
 
   // annotations
-  pageDict->lookupNF("Annots", &annots);
-  if (!(annots.isRef() || annots.isArray() || annots.isNull())) {
+  pageDict->lookupNF("Annots", &annotsObj);
+  if (!(annotsObj.isRef() || annotsObj.isArray() || annotsObj.isNull())) {
     error(-1, "Page annotations object (page %d) is wrong type (%s)",
-	  num, annots.getTypeName());
-    annots.free();
+	  num, annotsObj.getTypeName());
+    annotsObj.free();
     goto err2;
   }
 
-  // forms
-  pageWidgets = new FormPageWidgets(xrefA, this->getAnnots(&tmp),num,form);
-  tmp.free();
-
   // contents
   pageDict->lookupNF("Contents", &contents);
   if (!(contents.isRef() || contents.isArray() ||
@@ -331,17 +325,17 @@ Page::Page(XRef *xrefA, int numA, Dict *pageDict, Ref pageRefA, PageAttrs *attrs
 
   trans.initNull();
  err2:
-  annots.initNull();
+  annotsObj.initNull();
  err1:
   contents.initNull();
   ok = gFalse;
 }
 
 Page::~Page() {
-  delete pageWidgets;
   delete attrs;
+  delete annots;
   pageObj.free();
-  annots.free();
+  annotsObj.free();
   contents.free();
   trans.free();
   thumb.free();
@@ -349,20 +343,26 @@ Page::~Page() {
 }
 
 Annots *Page::getAnnots(Catalog *catalog) {
-  Annots *annots;
-  Object obj;
+  if (!annots) {
+    Object obj;
+    annots = new Annots(xref, catalog, getAnnots(&obj));
+    obj.free();
+  }
 
-  annots = new Annots(xref, catalog, getAnnots(&obj));
-  obj.free();
   return annots;
 }
 
-void Page::addAnnot(Annot *annot) {
+void Page::addAnnot(Annot *annot, Catalog *catalog) {
   Object obj1;
   Object tmp;
   Ref annotRef = annot->getRef ();
 
-  if (annots.isNull()) {
+  // Make sure we have annots before adding the new one
+  // even if it's an empty list so that we can safely
+  // call annots->appendAnnot(annot)
+  getAnnots(catalog);
+
+  if (annotsObj.isNull()) {
     Ref annotsRef;
     // page doesn't have annots array,
     // we have to create it
@@ -372,26 +372,32 @@ void Page::addAnnot(Annot *annot) {
     tmp.free();
 
     annotsRef = xref->addIndirectObject (&obj1);
-    annots.initRef(annotsRef.num, annotsRef.gen);
-    pageObj.dictSet ("Annots", &annots);
+    annotsObj.initRef(annotsRef.num, annotsRef.gen);
+    pageObj.dictSet ("Annots", &annotsObj);
     xref->setModifiedObject (&pageObj, pageRef);
   } else {
     getAnnots(&obj1);
     if (obj1.isArray()) {
       obj1.arrayAdd (tmp.initRef (annotRef.num, annotRef.gen));
-      xref->setModifiedObject (&obj1, annots.getRef());
+      if (annotsObj.isRef())
+        xref->setModifiedObject (&obj1, annotsObj.getRef());
+      else
+        xref->setModifiedObject (&pageObj, pageRef);
     }
     obj1.free();
   }
+
+  annots->appendAnnot(annot);
+
+  annot->setPage(&pageRef, num);
 }
 
 Links *Page::getLinks(Catalog *catalog) {
-  Links *links;
-  Object obj;
+  return new Links(getAnnots(catalog));
+}
 
-  links = new Links(getAnnots(&obj), catalog->getBaseURI());
-  obj.free();
-  return links;
+FormPageWidgets *Page::getFormWidgets(Catalog *catalog) {
+  return new FormPageWidgets(getAnnots(catalog), num, catalog->getForm());
 }
 
 void Page::display(OutputDev *out, double hDPI, double vDPI,
@@ -480,9 +486,8 @@ void Page::displaySlice(OutputDev *out, double hDPI, double vDPI,
   obj.free();
 
   // draw annotations
-  annotList = new Annots(xref, catalog, getAnnots(&obj));
-  obj.free();
-  
+  annotList = getAnnots(catalog);
+
   if (annotList->getNumAnnots() > 0) {
     if (globalParams->getPrintCommands()) {
       printf("***** Annotations\n");
@@ -497,7 +502,6 @@ void Page::displaySlice(OutputDev *out, double hDPI, double vDPI,
     }
     out->dump();
   }
-  delete annotList;
 
   delete gfx;
 }
