@@ -10,6 +10,9 @@
 //
 // Modified under the Poppler project - http://poppler.freedesktop.org
 //
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
 // Copyright (C) 2006, 2007 Ed Catmur <ed@catmur.co.uk>
 // Copyright (C) 2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2008 Albert Astals Cid <aacid@kde.org>
@@ -1167,8 +1170,11 @@ Unicode *unicodeNormalizeNFKC(Unicode *in, int len,
 	    break;
 	  u = out[r]; out[r] = out[r - 1]; out[r - 1] = u;
 	  swap = classes[r]; classes[r] = classes[r - 1]; classes[r - 1] = swap;
-	  if (indices)
-	    swap = idx[r]; idx[r] = idx[r - 1]; idx[r - 1] = swap;
+	  if (indices) {
+	    swap = idx[r]; 
+	    idx[r] = idx[r - 1];
+	    idx[r - 1] = swap;
+	  }
 	}
       // canonical compose out[o, p)
       for (q = o + 1; q < p; ++q)
