@@ -1,5 +1,7 @@
 /* poppler-form.h: qt4 interface to poppler
- * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
+ * Copyright (C) 2007-2008, 2011, Pino Toscano <pino@kde.org>
+ * Copyright (C) 2008, 2011, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2011 Carlos Garcia Campos <carlosgc@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +24,7 @@
 
 #include <Form.h>
 #include <Object.h>
+#include <Link.h>
 
 #include "poppler-form.h"
 #include "poppler-page-private.h"
@@ -30,6 +33,35 @@
 
 #include <math.h>
 
+namespace {
+
+Qt::Alignment formTextAlignment(Object *obj)
+{
+  Object tmp;
+  int align = 0;
+  if (obj->dictLookup("Q", &tmp)->isInt())
+  {
+    align = tmp.getInt();
+  }
+  tmp.free();
+  Qt::Alignment qtalign;
+  switch (align)
+  {
+    case 1:
+      qtalign = Qt::AlignHCenter;
+      break;
+    case 2:
+      qtalign = Qt::AlignRight;
+      break;
+    case 0:
+    default:
+      qtalign = Qt::AlignLeft;
+  }
+  return qtalign;
+}
+
+}
+
 namespace Poppler {
 
 FormField::FormField(FormFieldData &dd)
@@ -40,7 +72,7 @@ FormField::FormField(FormFieldData &dd)
   double left, top, right, bottom;
   m_formData->fm->getRect(&left, &bottom, &right, &top);
   // build a normalized transform matrix for this page at 100% scale
-  GfxState gfxState( 72.0, 72.0, m_formData->page->getMediaBox(), rotation, gTrue );
+  GfxState gfxState( 72.0, 72.0, m_formData->page->getCropBox(), rotation, gTrue );
   double * gfxCTM = gfxState.getCTM();
   double MTX[6];
   double pageWidth = m_formData->page->getCropWidth();
@@ -59,17 +91,17 @@ FormField::FormField(FormFieldData &dd)
   XPDFReader::transform( MTX, qMax( left, right ), qMin( top, bottom ), bottomRight );
   m_formData->box = QRectF(topLeft, QSizeF(bottomRight.x() - topLeft.x(), bottomRight.y() - topLeft.y()));
 
-  Object *obj = m_formData->fm->getObj();
+  Dict *dict = m_formData->fm->getObj()->getDict();
   Object tmp;
 
   // reading the flags
-  if (obj->isDict() && obj->dictLookup("Ff", &tmp)->isInt())
+  if (dict->lookup("Ff", &tmp)->isInt())
   {
     m_formData->flags = tmp.getInt();
   }
   tmp.free();
   // reading the widget annotation flags
-  if (obj->isDict() && obj->dictLookup("F", &tmp)->isInt())
+  if (dict->lookup("F", &tmp)->isInt())
   {
     m_formData->annoflags = tmp.getInt();
   }
@@ -94,31 +126,31 @@ int FormField::id() const
 
 QString FormField::name() const
 {
-  Object tmp;
-  Object *obj = m_formData->fm->getObj();
   QString name;
-  if (obj->dictLookup("T", &tmp)->isString())
+  if (GooString *goo = m_formData->fm->getPartialName())
   {
-    GooString *goo = tmp.getString();
-    if (goo)
-      name = goo->getCString();
+    name = QString::fromLatin1(goo->getCString());
+  }
+  return name;
+}
+
+QString FormField::fullyQualifiedName() const
+{
+  QString name;
+  if (GooString *goo = m_formData->fm->getFullyQualifiedName())
+  {
+    name = QString::fromLatin1(goo->getCString());
   }
-  tmp.free();
   return name;
 }
 
 QString FormField::uiName() const
 {
-  Object tmp;
-  Object *obj = m_formData->fm->getObj();
   QString name;
-  if (obj->dictLookup("TU", &tmp)->isString())
+  if (GooString *goo = m_formData->fm->getAlternateUiName())
   {
-    GooString *goo = tmp.getString();
-    if (goo)
-      name = goo->getCString();
+    name = QString::fromLatin1(goo->getCString());
   }
-  tmp.free();
   return name;
 }
 
@@ -134,19 +166,11 @@ bool FormField::isVisible() const
 
 Link* FormField::activationAction() const
 {
-  Object tmp;
-  Object *obj = m_formData->fm->getObj();
   Link* action = 0;
-  if (obj->dictLookup("A", &tmp)->isDict())
+  if (::LinkAction *act = m_formData->fm->getActivationAction())
   {
-    ::LinkAction *act = ::LinkAction::parseAction(&tmp, m_formData->doc->doc->getCatalog()->getBaseURI());
-    if (act)
-    {
-      action = PageData::convertLinkActionToLink(act, m_formData->doc, QRectF());
-      delete act;
-    }
+    action = PageData::convertLinkActionToLink(act, m_formData->doc, QRectF());
   }
-  tmp.free();
   return action;
 }
 
@@ -186,11 +210,29 @@ FormFieldButton::ButtonType FormFieldButton::buttonType() const
 QString FormFieldButton::caption() const
 {
   FormWidgetButton* fwb = static_cast<FormWidgetButton*>(m_formData->fm);
-  // HACK push buttons seems to have a null GooString for the caption
+  QString ret;
   if (fwb->getButtonType() == formButtonPush)
-    return QString();
-
-  return fwb->getOnStr() ? QString::fromUtf8(fwb->getOnStr()) : QString();
+  {
+    Dict *dict = m_formData->fm->getObj()->getDict();
+    Object obj1;
+    if (dict->lookup("MK", &obj1)->isDict())
+    {
+      AnnotAppearanceCharacs appearCharacs(obj1.getDict());
+      if (appearCharacs.getNormalCaption())
+      {
+        ret = UnicodeParsedString(appearCharacs.getNormalCaption());
+      }
+    }
+    obj1.free();
+  }
+  else
+  {
+    if (const char *goo = fwb->getOnStr())
+    {
+      ret = QString::fromUtf8(goo);
+    }
+  }
+  return ret;
 }
 
 bool FormFieldButton::state() const
@@ -279,7 +321,7 @@ int FormFieldText::maximumLength() const
 
 Qt::Alignment FormFieldText::textAlignment() const
 {
-  return m_formData->textAlignment(m_formData->fm->getObj());
+  return formTextAlignment(m_formData->fm->getObj());
 }
 
 bool FormFieldText::canBeSpellChecked() const
@@ -357,7 +399,7 @@ void FormFieldChoice::setCurrentChoices( const QList<int> &choice )
 
 Qt::Alignment FormFieldChoice::textAlignment() const
 {
-  return m_formData->textAlignment(m_formData->fm->getObj());
+  return formTextAlignment(m_formData->fm->getObj());
 }
 
 bool FormFieldChoice::canBeSpellChecked() const
