@@ -3,7 +3,7 @@
  * Copyright (C) 2005, Brad Hards <bradh@frogmouth.net>
  * Copyright (C) 2005-2011, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2005, Stefan Kebekus <stefan.kebekus@math.uni-koeln.de>
- * Copyright (C) 2006-2010, Pino Toscano <pino@kde.org>
+ * Copyright (C) 2006-2011, Pino Toscano <pino@kde.org>
  * Copyright (C) 2008 Carlos Garcia Campos <carlosgc@gnome.org>
  * Copyright (C) 2009 Shawn Rutledge <shawn.t.rutledge@gmail.com>
  * Copyright (C) 2010, Guillermo Amaral <gamaral@kdab.com>
@@ -42,6 +42,7 @@
 #include <TextOutputDev.h>
 #include <Annot.h>
 #include <Link.h>
+#include <FileSpec.h>
 #include <ArthurOutputDev.h>
 #if defined(HAVE_SPLASH)
 #include <SplashOutputDev.h>
@@ -558,7 +559,6 @@ QList<Annotation*> Page::annotations() const
     const uint numAnnotations = annots->getNumAnnots();
     if ( numAnnotations == 0 )
     {
-        delete annots;
         return QList<Annotation*>();
     }
 
@@ -907,11 +907,9 @@ QList<Annotation*> Page::annotations() const
                 // TODO
 
                 // reading link action
-                if ( !linkann->getDest()->isNull() )
+                if ( linkann->getAction() )
                 {
-                    ::LinkAction *act = ::LinkAction::parseDest( linkann->getDest() );
-                    Link * popplerLink = m_page->convertLinkActionToLink( act, QRectF() );
-                    delete act;
+                    Link * popplerLink = m_page->convertLinkActionToLink( linkann->getAction(), QRectF() );
                     if ( popplerLink )
                     {
                         l->setLinkDestination( popplerLink );
@@ -942,8 +940,8 @@ QList<Annotation*> Page::annotations() const
                 // -> fileIcon
                 f->setFileIconName( QString::fromLatin1( attachann->getName()->getCString() ) );
                 // -> embeddedFile
-                EmbFile *embfile = new EmbFile( attachann->getFile(), attachann->getContents() );
-                f->setEmbeddedFile( new EmbeddedFile( embfile ) );
+                FileSpec *filespec = new FileSpec( attachann->getFile() );
+                f->setEmbeddedFile( new EmbeddedFile( *new EmbeddedFileData( filespec ) ) );
                 break;
             }
             case Annot::typeSound:
@@ -980,6 +978,9 @@ QList<Annotation*> Page::annotations() const
             // special case for ignoring unknwon annotations
             case Annot::typeUnknown:
                 continue;
+            // handled as forms or some other way
+            case Annot::typeWidget:
+                continue;
             default:
             {
 #define CASE_FOR_TYPE( thetype ) \
@@ -989,7 +990,6 @@ QList<Annotation*> Page::annotations() const
                 QByteArray type;
                 switch ( subType )
                 {
-                    CASE_FOR_TYPE( Widget )
                     CASE_FOR_TYPE( Screen )
                     CASE_FOR_TYPE( PrinterMark )
                     CASE_FOR_TYPE( TrapNet )
@@ -1301,7 +1301,6 @@ QList<Annotation*> Page::annotations() const
         }
     }
 
-    delete annots;
     /** 5 - finally RETURN ANNOTATIONS */
     return annotationsMap.values();
 }
@@ -1310,7 +1309,7 @@ QList<FormField*> Page::formFields() const
 {
   QList<FormField*> fields;
   ::Page *p = m_page->page;
-  ::FormPageWidgets * form = p->getPageWidgets();
+  ::FormPageWidgets * form = p->getFormWidgets(m_page->parentDoc->doc->getCatalog());
   int formcount = form->getNumWidgets();
   for (int i = 0; i < formcount; ++i)
   {
@@ -1343,6 +1342,8 @@ QList<FormField*> Page::formFields() const
       fields.append(ff);
   }
 
+  delete form;
+
   return fields;
 }
 
