@@ -1,6 +1,7 @@
 #include <QtTest/QtTest>
 
 #include "PDFDoc.h"
+#include "GlobalParams.h"
 
 #include <poppler-qt4.h>
 
@@ -85,6 +86,7 @@ void TestOptionalContent::checkNoOptionalContent()
 void TestOptionalContent::checkIsVisible()
 {
     GooString *fileName = new GooString("../../../test/unittestcases/vis_policy_test.pdf"); 
+    globalParams = new GlobalParams();
     PDFDoc *doc = new PDFDoc( fileName );
     QVERIFY( doc );
 
@@ -161,10 +163,12 @@ void TestOptionalContent::checkIsVisible()
     obj.free();
 
     delete doc;
+    delete globalParams;
 }
 
 void TestOptionalContent::checkVisibilitySetting()
 {
+    globalParams = new GlobalParams();
     GooString *fileName = new GooString("../../../test/unittestcases/vis_policy_test.pdf"); 
     PDFDoc *doc = new PDFDoc( fileName );
     QVERIFY( doc );
@@ -184,8 +188,8 @@ void TestOptionalContent::checkVisibilitySetting()
     OptionalContentGroup *ocgA = ocgs->findOcgByRef( ref21 );
     QVERIFY( ocgA );
 
-    QVERIFY( (ocgA->name()->cmp("A")) == 0 );
-    QCOMPARE( ocgA->state(), OptionalContentGroup::On );
+    QVERIFY( (ocgA->getName()->cmp("A")) == 0 );
+    QCOMPARE( ocgA->getState(), OptionalContentGroup::On );
 
     Object ref28obj;
     ref28obj.initRef( 28, 0 );
@@ -193,8 +197,8 @@ void TestOptionalContent::checkVisibilitySetting()
     OptionalContentGroup *ocgB = ocgs->findOcgByRef( ref28 );
     QVERIFY( ocgB );
 
-    QVERIFY( (ocgB->name()->cmp("B")) == 0 );
-    QCOMPARE( ocgB->state(), OptionalContentGroup::On );
+    QVERIFY( (ocgB->getName()->cmp("B")) == 0 );
+    QCOMPARE( ocgB->getState(), OptionalContentGroup::On );
 
     // turn one Off
     ocgA->setState( OptionalContentGroup::Off );
@@ -394,6 +398,7 @@ void TestOptionalContent::checkVisibilitySetting()
     obj.free();
 
     delete doc;
+    delete globalParams;
 }
 
 void TestOptionalContent::checkRadioButtons()
@@ -425,7 +430,7 @@ void TestOptionalContent::checkRadioButtons()
     QCOMPARE( static_cast<Qt::CheckState>( optContent->data( subindex, Qt::CheckStateRole ).toInt() ), Qt::Unchecked );
 
     // RBGroup of languages, so turning on Japanese should turn off English
-    bool result = optContent->setData( subindex, QVariant( true ), Qt::CheckStateRole );
+    QVERIFY( optContent->setData( subindex, QVariant( true ), Qt::CheckStateRole ) );
 
     subindex = optContent->index( 0, 0, index );
     QCOMPARE( optContent->data( subindex, Qt::DisplayRole ).toString(), QString( "English" ) );
@@ -440,7 +445,7 @@ void TestOptionalContent::checkRadioButtons()
     QCOMPARE( static_cast<Qt::CheckState>( optContent->data( subindex, Qt::CheckStateRole ).toInt() ), Qt::Unchecked );
 
     // and turning on French should turn off Japanese
-    result = optContent->setData( subindex, QVariant( true ), Qt::CheckStateRole );
+    QVERIFY( optContent->setData( subindex, QVariant( true ), Qt::CheckStateRole ) );
 
     subindex = optContent->index( 0, 0, index );
     QCOMPARE( optContent->data( subindex, Qt::DisplayRole ).toString(), QString( "English" ) );
@@ -456,7 +461,7 @@ void TestOptionalContent::checkRadioButtons()
 
 
     // and turning off French should leave them all off
-    result = optContent->setData( subindex, QVariant( false ), Qt::CheckStateRole );
+    QVERIFY( optContent->setData( subindex, QVariant( false ), Qt::CheckStateRole ) );
 
     subindex = optContent->index( 0, 0, index );
     QCOMPARE( optContent->data( subindex, Qt::DisplayRole ).toString(), QString( "English" ) );
