@@ -13,6 +13,8 @@
 //
 // Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
 // Copyright (C) 2009 Petr Gajdos <pgajdos@novell.com>
+// Copyright (C) 2009, 2011 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2011 Andreas Hartmetz <ahartmetz@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -27,6 +29,7 @@
 #endif
 
 #include "goo/gtypes.h"
+#include "SplashTypes.h"
 
 class SplashT1FontEngine;
 class SplashFTFontEngine;
@@ -55,7 +58,8 @@ public:
 #endif
 #if HAVE_FREETYPE_FREETYPE_H || HAVE_FREETYPE_H
 		   GBool enableFreeType,
-		   GBool noah,
+		   GBool enableFreeTypeHinting,
+		   GBool enableSlightHinting,
 #endif
 		   GBool aa);
 
