@@ -1,4 +1,36 @@
+//========================================================================
+//
+// This file comes from pdftohtml project
+// http://pdftohtml.sourceforge.net
+//
+// Copyright from:
+// Gueorgui Ovtcharov
+// Rainer Dorsch <http://www.ra.informatik.uni-stuttgart.de/~rainer/>
+// Mikhail Kruk <meshko@cs.brandeis.edu>
+//
+//========================================================================
+
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2007, 2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008 Boris Toloknov <tlknv@yandex.ru>
+// Copyright (C) 2008 Tomas Are Haavet <tomasare@gmail.com>
+// Copyright (C) 2010 OSSD CDAC Mumbai by Leena Chourey (leenac@cdacmumbai.in) and Onkar Potdar (onkar@cdacmumbai.in)
+// Copyright (C) 2011 Joshua Richardson <jric@chegg.com>
+// Copyright (C) 2011 Stephen Reichling <sreichling@chegg.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include "HtmlFonts.h"
+#include "HtmlUtils.h"
 #include "GlobalParams.h"
 #include "UnicodeMap.h"
 #include <stdio.h>
@@ -90,6 +122,7 @@ HtmlFont::HtmlFont(GooString* ftname,int _size, GfxRGB rgb){
   size=(_size-1);
   italic = gFalse;
   bold = gFalse;
+  rotOrSkewed = gFalse;
 
   if (fontname){
     if (strstr(fontname->lowerCase()->getCString(),"bold"))  bold=gTrue;
@@ -104,7 +137,8 @@ HtmlFont::HtmlFont(GooString* ftname,int _size, GfxRGB rgb){
 	}
     pos=i;
     delete fontname;
-  }  
+  } else
+    pos = font_num; 
   if (!DefaultFont) DefaultFont=new GooString(fonts[font_num].name);
 
 }
@@ -117,6 +151,8 @@ HtmlFont::HtmlFont(const HtmlFont& x){
    pos=x.pos;
    color=x.color;
    if (x.FontName) FontName=new GooString(x.FontName);
+   rotOrSkewed = x.rotOrSkewed;
+   memcpy(rotSkewMat, x.rotSkewMat, sizeof(rotSkewMat));
  }
 
 
@@ -145,14 +181,15 @@ void HtmlFont::clear(){
 
 
 /*
-  This function is used to compare font uniquily for insertion into
+  This function is used to compare font uniquely for insertion into
   the list of all encountered fonts
 */
 GBool HtmlFont::isEqual(const HtmlFont& x) const{
-  return ((size==x.size) &&
+  return (size==x.size) &&
 	  (lineSize==x.lineSize) &&
 	  (pos==x.pos) && (bold==x.bold) && (italic==x.italic) &&
-	  (color.isEqual(x.getColor())));
+	  (color.isEqual(x.getColor())) && isRotOrSkewed() == x.isRotOrSkewed() &&
+	  (!isRotOrSkewed() || rot_matrices_equal(getRotMat(), x.getRotMat()));
 }
 
 /*
@@ -201,11 +238,11 @@ GooString* HtmlFont::HtmlFilter(Unicode* u, int uLen) {
   for (int i = 0; i < uLen; ++i) {
     switch (u[i])
       { 
-	case '"': tmp->append("&quot;");  break;
+	case '"': tmp->append("&#34;");  break;
 	case '&': tmp->append("&amp;");  break;
 	case '<': tmp->append("&lt;");  break;
 	case '>': tmp->append("&gt;");  break;
-	case ' ': tmp->append( !xml && ( i+1 >= uLen || !tmp->getLength() || tmp->getChar( tmp->getLength()-1 ) == ' ' ) ? "&nbsp;" : " " );
+	case ' ': tmp->append( !xml && ( i+1 >= uLen || !tmp->getLength() || tmp->getChar( tmp->getLength()-1 ) == ' ' ) ? "&#160;" : " " );
 	          break;
 	default:  
 	  {
@@ -237,7 +274,7 @@ GooString* HtmlFont::simple(HtmlFont* font, Unicode* content, int uLen){
 }
 
 HtmlFontAccu::HtmlFontAccu(){
-  accu=new GooVector<HtmlFont>();
+  accu=new std::vector<HtmlFont>();
 }
 
 HtmlFontAccu::~HtmlFontAccu(){
@@ -245,7 +282,7 @@ HtmlFontAccu::~HtmlFontAccu(){
 }
 
 int HtmlFontAccu::AddFont(const HtmlFont& font){
- GooVector<HtmlFont>::iterator i; 
+ std::vector<HtmlFont>::iterator i; 
  for (i=accu->begin();i!=accu->end();i++)
  {
 	if (font.isEqual(*i)) 
@@ -258,32 +295,13 @@ int HtmlFontAccu::AddFont(const HtmlFont& font){
  return (accu->size()-1);
 }
 
-// get CSS font name for font #i 
-GooString* HtmlFontAccu::getCSStyle(int i, GooString* content){
-  GooString *tmp;
-  GooString *iStr=GooString::fromInt(i);
-  
-  if (!xml) {
-    tmp = new GooString("<span class=\"ft");
-    tmp->append(iStr);
-    tmp->append("\">");
-    tmp->append(content);
-    tmp->append("</span>");
-  } else {
-    tmp = new GooString("");
-    tmp->append(content);
-  }
-
-  delete iStr;
-  return tmp;
-}
-
 // get CSS font definition for font #i 
-GooString* HtmlFontAccu::CSStyle(int i){
+GooString* HtmlFontAccu::CSStyle(int i, int j){
    GooString *tmp=new GooString();
    GooString *iStr=GooString::fromInt(i);
+   GooString *jStr=GooString::fromInt(j);
 
-   GooVector<HtmlFont>::iterator g=accu->begin();
+   std::vector<HtmlFont>::iterator g=accu->begin();
    g+=i;
    HtmlFont font=*g;
    GooString *Size=GooString::fromInt(font.getSize());
@@ -293,10 +311,11 @@ GooString* HtmlFontAccu::CSStyle(int i){
    
    if(!xml){
      tmp->append(".ft");
+     tmp->append(jStr);
      tmp->append(iStr);
      tmp->append("{font-size:");
      tmp->append(Size);
-     if( font.getLineSize() != -1 )
+     if( font.getLineSize() != -1 && font.getLineSize() != 0 )
      {
 	 lSize = GooString::fromInt(font.getLineSize());
 	 tmp->append("px;line-height:");
@@ -307,6 +326,29 @@ GooString* HtmlFontAccu::CSStyle(int i){
      tmp->append(fontName); //font.getFontName());
      tmp->append(";color:");
      tmp->append(colorStr);
+     // if there is rotation or skew, include the matrix
+     if (font.isRotOrSkewed()) {
+    	 const double * const text_mat = font.getRotMat();
+    	 GooString matrix_str(" matrix(");
+    	 matrix_str.appendf("{0:10.10g}, {1:10.10g}, {2:10.10g}, {3:10.10g}, 0, 0)",
+    			 text_mat[0], text_mat[1], text_mat[2], text_mat[3]);
+    	 tmp->append(";-moz-transform:");
+    	 tmp->append(&matrix_str);
+    	 tmp->append(";-webkit-transform:");
+    	 tmp->append(&matrix_str);
+    	 tmp->append(";-o-transform:");
+    	 tmp->append(&matrix_str);
+    	 tmp->append(";-ms-transform:");
+    	 tmp->append(&matrix_str);
+    	 // Todo: 75% is a wild guess that seems to work pretty well;
+    	 // We probably need to calculate the real percentage
+    	 // Based on the characteristic baseline and bounding box of current font
+    	 // PDF origin is at baseline
+    	 tmp->append(";-moz-transform-origin: left 75%");
+    	 tmp->append(";-webkit-transform-origin: left 75%");
+    	 tmp->append(";-o-transform-origin: left 75%");
+    	 tmp->append(";-ms-transform-origin: left 75%");
+     }
      tmp->append(";}");
    }
    if (xml) {
@@ -323,6 +365,7 @@ GooString* HtmlFontAccu::CSStyle(int i){
 
    delete fontName;
    delete colorStr;
+   delete jStr;
    delete iStr;
    delete Size;
    return tmp;
