@@ -17,7 +17,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005-2011 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2012 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2008 Kjartan Maraas <kmaraas@gnome.org>
 // Copyright (C) 2008 Boris Toloknov <tlknv@yandex.ru>
 // Copyright (C) 2008 Haruyuki Kawabe <Haruyuki.Kawabe@unisys.co.jp>
@@ -30,6 +30,7 @@
 // Copyright (C) 2010 OSSD CDAC Mumbai by Leena Chourey (leenac@cdacmumbai.in) and Onkar Potdar (onkar@cdacmumbai.in)
 // Copyright (C) 2011 Joshua Richardson <jric@chegg.com>
 // Copyright (C) 2011 Stephen Reichling <sreichling@chegg.com>
+// Copyright (C) 2012 Igor Slepchin <igor.redhat@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -61,6 +62,8 @@
 #include "HtmlOutputDev.h"
 #include "HtmlFonts.h"
 #include "HtmlUtils.h"
+#include "Outline.h"
+#include "PDFDoc.h"
 
 #define DEBUG __FILE__ << ": " << __LINE__ << ": DEBUG: "
 
@@ -739,7 +742,6 @@ void HtmlPage::dumpAsXML(FILE* f,int page){
 
 int HtmlPage::dumpComplexHeaders(FILE * const file, FILE *& pageFile, int page) {
   GooString* tmp;
-  char* htmlEncoding;
 
   if( !noframes )
   {
@@ -766,11 +768,12 @@ int HtmlPage::dumpComplexHeaders(FILE * const file, FILE *& pageFile, int page)
 
       delete tmp;
 
-      htmlEncoding = HtmlOutputDev::mapEncodingToHtml(globalParams->getTextEncodingName());
+      GooString *htmlEncoding = HtmlOutputDev::mapEncodingToHtml(globalParams->getTextEncodingName());
       if (!singleHtml)
-        fprintf(pageFile, "<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n", htmlEncoding);
+        fprintf(pageFile, "<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n", htmlEncoding->getCString());
       else
-        fprintf(pageFile, "<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n <br/>\n", htmlEncoding);
+        fprintf(pageFile, "<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n <br/>\n", htmlEncoding->getCString());
+      delete htmlEncoding;
   }
   else 
   {
@@ -956,23 +959,23 @@ static char* HtmlEncodings[][2] = {
     {NULL, NULL}
 };
 
-
-char* HtmlOutputDev::mapEncodingToHtml(GooString* encoding)
+GooString* HtmlOutputDev::mapEncodingToHtml(GooString* encoding)
 {
-    char* enc = encoding->getCString();
-    for(int i = 0; HtmlEncodings[i][0] != NULL; i++)
+  GooString* enc = encoding;
+  for(int i = 0; HtmlEncodings[i][0] != NULL; i++)
+  {
+    if( enc->cmp(HtmlEncodings[i][0]) == 0 )
     {
-	if( strcmp(enc, HtmlEncodings[i][0]) == 0 )
-	{
-	    return HtmlEncodings[i][1];
-	}
+      delete enc;
+      return new GooString(HtmlEncodings[i][1]);
     }
-    return enc; 
+  }
+  return enc; 
 }
 
 void HtmlOutputDev::doFrame(int firstPage){
   GooString* fName=new GooString(Docname);
-  char* htmlEncoding;
+  GooString* htmlEncoding;
   fName->append(".html");
 
   if (!(fContentsFrame = fopen(fName->getCString(), "w"))){
@@ -989,7 +992,7 @@ void HtmlOutputDev::doFrame(int firstPage){
   fputs("\n<HEAD>",fContentsFrame);
   fprintf(fContentsFrame,"\n<TITLE>%s</TITLE>",docTitle->getCString());
   htmlEncoding = mapEncodingToHtml(globalParams->getTextEncodingName());
-  fprintf(fContentsFrame, "\n<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n", htmlEncoding);
+  fprintf(fContentsFrame, "\n<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n", htmlEncoding->getCString());
   dumpMetaVars(fContentsFrame);
   fprintf(fContentsFrame, "</HEAD>\n");
   fputs("<FRAMESET cols=\"100,*\">\n",fContentsFrame);
@@ -1003,6 +1006,7 @@ void HtmlOutputDev::doFrame(int firstPage){
   fputs(">\n</FRAMESET>\n</HTML>\n",fContentsFrame);
  
   delete fName;
+  delete htmlEncoding;
   fclose(fContentsFrame);  
 }
 
@@ -1011,8 +1015,6 @@ HtmlOutputDev::HtmlOutputDev(char *fileName, char *title,
 	char *extension,
 	GBool rawOrder, int firstPage, GBool outline) 
 {
-  char *htmlEncoding;
-  
   fContentsFrame = NULL;
   docTitle = new GooString(title);
   pages = NULL;
@@ -1099,10 +1101,10 @@ HtmlOutputDev::HtmlOutputDev(char *fileName, char *title,
       delete right;
     }
 
-    htmlEncoding = mapEncodingToHtml(globalParams->getTextEncodingName()); 
+    GooString *htmlEncoding = mapEncodingToHtml(globalParams->getTextEncodingName()); 
     if (xml) 
     {
-      fprintf(page, "<?xml version=\"1.0\" encoding=\"%s\"?>\n", htmlEncoding);
+      fprintf(page, "<?xml version=\"1.0\" encoding=\"%s\"?>\n", htmlEncoding->getCString());
       fputs("<!DOCTYPE pdf2xml SYSTEM \"pdf2xml.dtd\">\n\n", page);
       fputs("<pdf2xml>\n",page);
     } 
@@ -1110,12 +1112,13 @@ HtmlOutputDev::HtmlOutputDev(char *fileName, char *title,
     {
       fprintf(page,"%s\n<HTML xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"\" xml:lang=\"\">\n<HEAD>\n<TITLE>%s</TITLE>\n", DOCTYPE, docTitle->getCString());
       
-      fprintf(page, "<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n", htmlEncoding);
+      fprintf(page, "<META http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"/>\n", htmlEncoding->getCString());
       
       dumpMetaVars(page);
       fprintf(page,"</HEAD>\n");
       fprintf(page,"<BODY bgcolor=\"#A0A0A0\" vlink=\"blue\" link=\"blue\">\n");
     }
+    delete htmlEncoding;
   }
   ok = gTrue; 
 }
@@ -1551,17 +1554,25 @@ void HtmlOutputDev::dumpMetaVars(FILE *file)
   }
 }
 
-GBool HtmlOutputDev::dumpDocOutline(Catalog* catalog)
+GBool HtmlOutputDev::dumpDocOutline(PDFDoc* doc)
 { 
+#ifdef DISABLE_OUTLINE
+	return gFalse;
+#else
 	FILE * output = NULL;
 	GBool bClose = gFalse;
+	Catalog *catalog = doc->getCatalog();
 
 	if (!ok || xml)
-    	return gFalse;
+                return gFalse;
   
-	Object *outlines = catalog->getOutline();
-  	if (!outlines->isDict())
-    	return gFalse;
+	Outline *outline = doc->getOutline();
+	if (!outline)
+		return gFalse;
+
+	GooList *outlines = outline->getItems();
+	if (!outlines)
+		return gFalse;
   
 	if (!complexMode && !xml)
   	{
@@ -1583,7 +1594,18 @@ GBool HtmlOutputDev::dumpDocOutline(Catalog* catalog)
 				return gFalse;
 			delete str;
 			bClose = gTrue;
-     		fputs("<HTML xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"\" xml:lang=\"\">\n<HEAD>\n<TITLE>Document Outline</TITLE>\n</HEAD>\n<BODY>\n", output);
+
+			GooString *htmlEncoding =
+				HtmlOutputDev::mapEncodingToHtml(globalParams->getTextEncodingName());
+
+			fprintf(output, "<HTML xmlns=\"http://www.w3.org/1999/xhtml\" " \
+                                "lang=\"\" xml:lang=\"\">\n"            \
+                                "<HEAD>\n"                              \
+                                "<TITLE>Document Outline</TITLE>\n"     \
+                                "<META http-equiv=\"Content-Type\" content=\"text/html; " \
+                                "charset=%s\"/>\n"                      \
+                                "</HEAD>\n<BODY>\n", htmlEncoding->getCString());
+			delete htmlEncoding;
 		}
 	}
  
@@ -1597,97 +1619,95 @@ GBool HtmlOutputDev::dumpDocOutline(Catalog* catalog)
 		fclose(output);
 	}
   	return done;
+#endif
 }
 
-GBool HtmlOutputDev::newOutlineLevel(FILE *output, Object *node, Catalog* catalog, int level)
+GBool HtmlOutputDev::newOutlineLevel(FILE *output, GooList *outlines, Catalog* catalog, int level)
 {
-  Object curr, next;
-  GBool atLeastOne = gFalse;
-  
-  if (node->dictLookup("First", &curr)->isDict()) {
-    if (level == 1)
+	GBool atLeastOne = gFalse;
+
+	if (level == 1)
 	{
 		fputs("<A name=\"outline\"></a>", output);
 		fputs("<h1>Document Outline</h1>\n", output);
 	}
-    fputs("<ul>",output);
-    do {
-      // get title, give up if not found
-      Object title;
-      if (curr.dictLookup("Title", &title)->isNull()) {
-		title.free();
-		break;
-      }
-      GooString *titleStr = new GooString(title.getString());
-      title.free();
+	fputs("<ul>\n",output);
 
-      // get corresponding link
-      // Note: some code duplicated from HtmlOutputDev::getLinkDest().
-      GooString *linkName = NULL;;
-      Object dest;
-      if (!curr.dictLookup("Dest", &dest)->isNull()) {
-		LinkGoTo *link = new LinkGoTo(&dest);
-		LinkDest *linkdest=NULL;
-		if (link->getDest()!=NULL)
-			linkdest=link->getDest()->copy();
-		else if (link->getNamedDest()!=NULL)
-			linkdest=catalog->findDest(link->getNamedDest());
-			
-		delete link;
-		if (linkdest) { 
-	  		int page;
-	  		if (linkdest->isPageRef()) {
-	    		Ref pageref=linkdest->getPageRef();
-	    		page=catalog->findPage(pageref.num,pageref.gen);
-	  		} else {
-	    		page=linkdest->getPageNum();
-	  		}
-	  		delete linkdest;
+	for (int i = 0; i < outlines->getLength(); i++)
+	{
+		OutlineItem *item = (OutlineItem*)outlines->get(i);
+		GooString *titleStr = HtmlFont::HtmlFilter(item->getTitle(),
+							   item->getTitleLength());
 
-			/* 			complex 	simple
-			frames		file-4.html	files.html#4
-			noframes	file.html#4	file.html#4
-	   		*/
-	  		linkName=basename(Docname);
-	  		GooString *str=GooString::fromInt(page);
-	  		if (noframes) {
-	    		linkName->append(".html#");
-				linkName->append(str);
-	  		} else {
-    			if( complexMode ) {
-	   		   		linkName->append("-");
-	      			linkName->append(str);
-	      			linkName->append(".html");
-	    		} else {
-	      			linkName->append("s.html#");
-	      			linkName->append(str);
-	    		}
-	  		}
-			delete str;
+		// get corresponding link
+		GooString *linkName = NULL;;
+		LinkAction *action = item->getAction();
+		LinkGoTo *link = NULL;
+		Object dest;
+		if (action && action->getKind() == actionGoTo)
+			link = dynamic_cast<LinkGoTo*>(action);
+		if (link && link->isOk()) {
+			LinkDest *linkdest=NULL;
+			if (link->getDest()!=NULL)
+				linkdest=link->getDest()->copy();
+			else if (link->getNamedDest()!=NULL)
+				linkdest=catalog->findDest(link->getNamedDest());
+
+			if (linkdest) {
+				int page;
+				if (linkdest->isPageRef()) {
+					Ref pageref=linkdest->getPageRef();
+					page=catalog->findPage(pageref.num,pageref.gen);
+				} else {
+					page=linkdest->getPageNum();
+				}
+				delete linkdest;
+
+				/*		complex		simple
+				frames		file-4.html	files.html#4
+				noframes	file.html#4	file.html#4
+				*/
+				linkName=basename(Docname);
+				GooString *str=GooString::fromInt(page);
+				if (noframes) {
+					linkName->append(".html#");
+					linkName->append(str);
+				} else {
+					if( complexMode ) {
+						linkName->append("-");
+						linkName->append(str);
+						linkName->append(".html");
+					} else {
+						linkName->append("s.html#");
+						linkName->append(str);
+					}
+				}
+				delete str;
+			}
 		}
-      }
-      dest.free();
+		dest.free();
 
-      fputs("<li>",output);
-      if (linkName)
-		fprintf(output,"<A href=\"%s\">", linkName->getCString());
-      fputs(titleStr->getCString(),output);
-      if (linkName) {
-		fputs("</A>",output);
-		delete linkName;
-      }
-      fputs("\n",output);
-      delete titleStr;
-      atLeastOne = gTrue;
+		fputs("<li>",output);
+		if (linkName)
+			fprintf(output,"<A href=\"%s\">", linkName->getCString());
+		fputs(titleStr->getCString(),output);
+		if (linkName) {
+			fputs("</A>",output);
+			delete linkName;
+		}
+		delete titleStr;
+		atLeastOne = gTrue;
 
-      newOutlineLevel(output, &curr, catalog, level+1);
-      curr.dictLookup("Next", &next);
-      curr.free();
-      curr = next;
-    } while(curr.isDict());
-    fputs("</ul>",output);
-  }
-  curr.free();
+		item->open();
+		if (item->hasKids())
+		{
+			fputs("\n",output);
+			newOutlineLevel(output, item->getKids(), catalog, level+1);
+		}
+		item->close();
+		fputs("</li>\n",output);
+	}
+	fputs("</ul>\n",output);
 
-  return atLeastOne;
+	return atLeastOne;
 }
