@@ -13,12 +13,13 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005, 2007 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2007, 2011 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006 Rainer Keller <class321@gmx.de>
 // Copyright (C) 2008 Timothy Lee <timothy.lee@siriushk.com>
 // Copyright (C) 2008 Vasile Gaburici <gaburici@cs.umd.edu>
 // Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 William Bader <williambader@hotmail.com>
+// Copyright (C) 2010 Jakob Voss <jakob.voss@gbv.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -41,16 +42,15 @@
 #include "GfxState.h"
 #include "Object.h"
 #include "Stream.h"
-#ifdef ENABLE_LIBJPEG
-#include "DCTStream.h"
-#endif
 #include "ImageOutputDev.h"
 
-ImageOutputDev::ImageOutputDev(char *fileRootA, GBool dumpJPEGA) {
+ImageOutputDev::ImageOutputDev(char *fileRootA, GBool pageNamesA, GBool dumpJPEGA) {
   fileRoot = copyString(fileRootA);
-  fileName = (char *)gmalloc(strlen(fileRoot) + 20);
+  fileName = (char *)gmalloc(strlen(fileRoot) + 45);
   dumpJPEG = dumpJPEGA;
+  pageNames = pageNamesA;
   imgNum = 0;
+  pageNum = 0;
   ok = gTrue;
 }
 
@@ -59,6 +59,14 @@ ImageOutputDev::~ImageOutputDev() {
   gfree(fileRoot);
 }
 
+void ImageOutputDev::setFilename(const char *fileExt) {
+  if (pageNames) {
+    sprintf(fileName, "%s-%03d-%03d.%s", fileRoot, pageNum, imgNum, fileExt);
+  } else {
+    sprintf(fileName, "%s-%03d.%s", fileRoot, imgNum, fileExt);
+  }
+}
+
 void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 				   int width, int height, GBool invert,
 				   GBool interpolate, GBool inlineImg) {
@@ -70,7 +78,7 @@ void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   if (dumpJPEG && str->getKind() == strDCT && !inlineImg) {
 
     // open the image file
-    sprintf(fileName, "%s-%03d.jpg", fileRoot, imgNum);
+    setFilename("jpg");
     ++imgNum;
     if (!(f = fopen(fileName, "wb"))) {
       error(-1, "Couldn't open image file '%s'", fileName);
@@ -78,7 +86,7 @@ void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
     }
 
     // initialize stream
-    str = ((DCTStream *)str)->getRawStream();
+    str = str->getNextStream();
     str->reset();
 
     // copy the stream
@@ -92,7 +100,7 @@ void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   } else {
 
     // open the image file and write the PBM header
-    sprintf(fileName, "%s-%03d.pbm", fileRoot, imgNum);
+    setFilename("pbm");
     ++imgNum;
     if (!(f = fopen(fileName, "wb"))) {
       error(-1, "Couldn't open image file '%s'", fileName);
@@ -137,7 +145,7 @@ void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
       !inlineImg) {
 
     // open the image file
-    sprintf(fileName, "%s-%03d.jpg", fileRoot, imgNum);
+    setFilename("jpg");
     ++imgNum;
     if (!(f = fopen(fileName, "wb"))) {
       error(-1, "Couldn't open image file '%s'", fileName);
@@ -145,7 +153,7 @@ void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
     }
 
     // initialize stream
-    str = ((DCTStream *)str)->getRawStream();
+    str = str->getNextStream();
     str->reset();
 
     // copy the stream
@@ -160,7 +168,7 @@ void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 	     colorMap->getBits() == 1) {
 
     // open the image file and write the PBM header
-    sprintf(fileName, "%s-%03d.pbm", fileRoot, imgNum);
+    setFilename("pbm");
     ++imgNum;
     if (!(f = fopen(fileName, "wb"))) {
       error(-1, "Couldn't open image file '%s'", fileName);
@@ -191,7 +199,7 @@ void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   } else {
 
     // open the image file and write the PPM header
-    sprintf(fileName, "%s-%03d.ppm", fileRoot, imgNum);
+    setFilename("ppm");
     ++imgNum;
     if (!(f = fopen(fileName, "wb"))) {
       error(-1, "Couldn't open image file '%s'", fileName);
